function [h] = gscatter(x,y,groups,clr,sym,siz,doleg,x_name,y_name)
// scatter plot of groups 
// Calling Sequence
//  gscatter(x,y,group)
//  gscatter(x,y,group,clr,sym,siz)
//  gscatter(x,y,group,clr,sym,siz,doleg)
//  gscatter(x,y,group,clr,sym,siz,doleg,xname,yname)
//  h = gscatter(...) 
// Parameters
//  x,y, group: 	vectors with equal length 
//  clr: 	color vector, default 'bgrcmyk'
//  sym:		symbol, default '.'
//  siz: 	size of Marker
//  doleg:  'on' (default) shows legend, 'off' turns of legend 
//  xname, yname: name of axis
//  Examples
//      loadmatfile("demos/data/iris.mat");
//      gscatter(meas(:,1),meas(:,2),species)
//
// See also
// ecdf
// cdfplot
// Authors
// 	Copyright (C) 2009 by Alois Schloegl a.schloegl@ieee.org
//  H. Nahrstaedt - 2010


//	$Id$
//	Copyright (C) 2009 by Alois Schloegl <a.schloegl@ieee.org>
//       This function is part of the NaN-toolbox
//       http://biosig-consulting.com/matlab/NaN/

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the  License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 [nargout,nargin]=argn(0);
if  ~isempty(groups)
   [groups] = mgrp2idx(groups,size(x,1),','); 
end
[b,j] = unique(groups); 

if nargin<3
	//help gscatter; 
	error('At least 3 arguments are needed!')
end;
if nargin<4, clr = [];end
if nargin<5, sym = [];end
if nargin<6, siz = [];end
if nargin<7, doleg = [];end
if nargin<8, x_name = [];end
if nargin<9, y_name = [];end; 

if isempty(clr), clr='bgrcmyk'; end; 
if isempty(sym), sym='.'; end; 
if isempty(doleg), doleg='on'; end; 



for k=1:length(b);
	//ix = find(k==j);
	c = part(clr,(mod(k-1,length(clr))+1)); 
	s = part(sym,(mod(k-1,length(sym))+1)); 
	plot(x(groups==b(k)),y(groups==b(k)),[c+s]);
        hh(k)=gce();
	if ~isempty(siz)	
		z = siz(mod(k-1,length(siz))+1); 
                e=gce();
                e.children(1).mark_size=z;
		//set(hh(k),'MarkerSize',z);
	end	
	//hold on; 
end; 
//hold off; 

if ~(doleg=='off')
// 	if or(type(b)==[1 5 8])
// 		b=string(b(:));
// 	end;	
	legend(string(b(:)));
end;
if ~isempty(x_name)
	xlabel(x_name); 
end; 
if ~isempty(y_name)
	ylabel(y_name); 
end; 
	
if nargout>0,
	h = hh;
end;
endfunction
function [ogroup,glabel,gname,multigroup,maxgroup] = mgrp2idx(groups,rows,sep)


[nargout,nargin]=argn(0);

multigroup = (iscell(groups) & size(groups,1)==1) |  (or(type(groups)==[1 4 8]) & ~isvector(groups) & ~isempty(groups));
if (~multigroup)
    [ogroup,gname] = grp2idx(groups);
    maxgroup = size(gname,'*');
    glabel = gname;
else
    // Group according to each distinct combination of grouping variables
    ngrps = size(groups,2);
    namemat = cell(1,ngrps);

    // Get integer codes and names for each grouping variable
    if iscell(groups)
        if ngrps == 0 // if no grouping vars, create one group containing all observations
            groups=cell(1,1);
            groups.entries(1) = ones(rows,1);
            ngrps = 1;
            namemat = cell(1,ngrps);
        end
        for j=1:ngrps
            [g,gn] = grp2idx(groups(1,j));
            if j==1
                rows = size(g,1);
                grpmat = zeros(rows,ngrps);
            elseif (size(g,1)~=rows)
                error( 'All grouping variables must have the same number of rows.');
            end
            grpmat(:,j) = g;
            namemat(1,j) = gn;
        end
    else
        rows = size(groups,1);
        grpmat = zeros(rows,ngrps);
        for j=1:ngrps
            [g,gn] = grp2idx(groups(:,j));
            grpmat(:,j) = g;
            namemat(1,j) = gn;
        end
    end
    
    // Find all unique combinations
    wasnan = or(isnan(grpmat),2);
    grpmat(wasnan,:) = [];
    [urows,ui,uj] = unique2(grpmat,'rows');
    
    // Create an index vector based on those unique combinations
    ogroup = mtlb_ones(size(wasnan))*%nan;
    ogroup(~wasnan) = uj;
    gname = cell(size(urows));

    for j=1:ngrps
        gn = namemat(1,j);
        gname(:,j) = gn(urows(:,j));
    end

    // Create another cell array of multi-line texts to use as labels
    glabel = cell(size(gname,1),1);
    if (nargin > 2)
        nl = sprintf(sep);
    else
        nl = sprintf('\n');
    end
    fmt = sprintf('%%s%s',nl);
    lnl = length(fmt)-3;        // one less than the length of nl
    for j=1:length(glabel)
        gn = sprintf(fmt, gname(j,:));
        gn($-lnl:$) = [];
        glabel(j,1) = gn;
    end
    maxgroup = size(glabel,'*');
end
endfunction